#
#   Raspberry Pi Meetup
#
#   Micropython code to Publish light sensor data to a Mosquitto broker using the MQTT protocol
#
#
# Hardware used:
# - Adafruit Huzzah ESP8266 Feather running micropython
# - Adafruit TSL2561 lux breakout board
# -- copy TSL2561 driver to board:  ampy -pCOMx -d1 put tsl2561.py
# - ssd1306 OLED display
#
#   Tested with Micropython release v1.9.3 (Nov 1, 2017)
#


# prerequisites:
# - Mosquitto MQTT server

import machine
import network
from umqtt.robust import MQTTClient
import time
import os
import tsl2561
import ssd1306 

#
#   blink LED for specified number of milliseconds
#
def blinkLed(blinkDurationInMs):
    builtInLed.off()
    time.sleep_ms(blinkDurationInMs)
    builtInLed.on()    
    
#
#   configure ESP8266 for I2C communication
#   GPIO4 pin is set to I2C SDA (data) 
#   GPIO5 pin is set to I2C SCL (clock) 
#
i2c = machine.I2C(scl=machine.Pin(5), sda=machine.Pin(4))

#
#   configure the built-in LED
#
builtInLed = machine.Pin(2, machine.Pin.OUT)
builtInLed.on()
blinkLed(500)

#   OLED display
oled = ssd1306.SSD1306_I2C(128, 32, i2c)

oled.fill(0)
oled.text('RPi Meetup', 0, 15)
oled.show()

#   light sensor
light_sensor = tsl2561.TSL2561(i2c)
light_sensor.gain(1)
light_sensor.integration_time(13)

#
#   WIFI connection information
#   
#   replace WIFI-SSID and WIFI-PASSWORD with access information for the WIFI network you will connect to
#
#wifiSSID = "WIFI-SSID"             # EDIT 1 - enter name of WiFi connection point
#wifiPassword = "WIFI-PASSWORD"     # EDIT 2 - enter WiFi password 

#
#   turn off the WiFi Access Point
# 
ap_if = network.WLAN(network.AP_IF)
ap_if.active(False)

#
#   create an interface that can connnect to a WIFI network
#
wifi = network.WLAN(network.STA_IF)

#
#   activate the WIFI interface
#
wifi.active(True)

#
#   connect the ESP8266 device to the WIFI network
#
wifi.connect(wifiSSID, wifiPassword)

#
#   wait until the ESP8266 has successfully connected to the WIFI network
# 
while not wifi.isconnected():
  pass
  
oled.fill(0)
oled.text('WiFi Connected', 0, 15)
oled.show()
  

blinkLed(1000)  
#
#   define IO connection information
#   create a random clientID 
#
randomNum = int.from_bytes(os.urandom(3), 'little')
myMqttClient = bytes("client"+str(randomNum), 'utf-8')

#
#   Edit MQTT server and topic
#
mosquittoUrl = b"<URL>"     # EDIT 3 - URL of Mosquitto server
mqttTopic = b"lux"                  # EDIT 4 - enter Topic name

#
#   configure a MQTT client
#
client = MQTTClient(client_id=myMqttClient, server=mosquittoUrl)#, port=1883)
                    
#
#   connect ESP8266 to Mosquitto broker IO using MQTT
#
client.connect()

#
#   continuously publish light sensor readings to Mosquitto broker using MQTT
#
while True:
    #
    #   read light from sensor
    #
    lux_reading = light_sensor.read()
    
    client.publish(mqttTopic, bytes(str(lux_reading), 'utf-8'))
    
    oled.fill(0)
    oled.text('{} lux'.format(lux_reading), 0, 0)
    oled.show()
    
    
    blinkLed(100)
    time.sleep(1)  # delay between each publish, in seconds

client.disconnect()  

